/*******************************************************************************
*  Config.cpp - Configure the program
*
*  Berzerk/
*
*  Part of the BuckoSoft Network Monitor
*  Copyright(c) 2009 - BuckoSoft, Inc.
*
*  http://git.buckosoft.com/gitweb/?p=berzerk.git;a=history;f=berzerk/Config.cpp
*
*  With which we configure Berzerk
\*******************************************************************************/

#include "stdafx.h"


#include <string>
#include <iostream>
#include <fstream>

#define	SYSLOG_NAMES
#include <syslog.h>

#include <boost/foreach.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/regex.hpp>

using namespace std;
using namespace boost;

#include "Config.h"
#include "Berzerk.h"

int Config::DEBUG = 0;

///////////////////////////////////////////////////////////////////////////////
Config::Config(void) {
	berzerk = NULL;
	threshold = 10;
	triggerDelay = 10;
	port = 9090;
	logFmt = "%F_%H:%M:%S";
}

///////////////////////////////////////////////////////////////////////////////
bool	Config::readFile(ifstream& ifs) {
	string		line;
	char		buff[256];

	bool	emptyLine = true;
	while(true) {
		ifs.getline(buff, 256);
		if (ifs.fail()) {
			break;
		}
		line = buff;
		if (DEBUG > 3)
			cout << "line: '" << line << "'" << endl;
		if (buff[0] == '#' || line.empty()) {
			emptyLine = true;
			continue;
		}
		typedef vector< string > split_vector_type;
		split_vector_type sv;
		split(sv, line, is_any_of("="));
		trim(sv[0]);
		trim(sv[1]);

		if (DEBUG > 4) {
			cout << "type ='" << sv[0] << "' data = '" << sv[1] << "'" << endl;
		}
		if (sv[0] == "threshold") {
			int i = atoi(sv[1].c_str());
			if (DEBUG > 4)
				cout << "++ threshold: '" << sv[1] << "' (" << i << ")" << endl;
			this->threshold = i;
			continue;
		}
		if (sv[0] == "triggerDelay") {
			int i = atoi(sv[1].c_str());
			if (DEBUG > 4)
				cout << "++ triggerDelay: '" << sv[1] << "' (" << i << ")" << endl;
			this->triggerDelay = i;
			continue;
		}
		if (sv[0] == "port") {
			int i = atoi(sv[1].c_str());
			if (DEBUG > 4)
				cout << "++ triggerDelay: '" << sv[1] << "' (" << i << ")" << endl;
			this->port = i;
			continue;
		}
		if (sv[0] == "whitelist") {
			if (DEBUG > 4)
				cout << "++ whitelist: '" << sv[1] << "'" << endl;
			this->addWhitelist(sv[1]);
			continue;
		}
		if (sv[0] == "logfmt") {
			if (DEBUG > 4)
				cout << "++ logfmt: '" << sv[1] << "'" << endl;
			this->logFmt = sv[1];
			continue;
		}
		if (sv[0] == "syslogfacility") {
			if (DEBUG > 4)
				cout << "++ syslogfacility: '" << sv[1] << "'" << endl;
			CODE* c = facilitynames;
			while (c->c_name != NULL) {
				if (!strcasecmp(sv[1].c_str(), c->c_name)) {
					this->setSyslogFacility(c->c_val);
					break;
				}
				c++;
			}
			if (c->c_name == NULL) {
				cout << "Unknown syslog facility";
				return(false);
			}
			continue;
		}
		if (sv[0] == "syslogpriority") {
			if (DEBUG > 4)
				cout << "++ syslogpriority: '" << sv[1] << "'" << endl;
			CODE* c = prioritynames;
			while (c->c_name != NULL) {
				if (!strcasecmp(sv[1].c_str(), c->c_name)) {
					this->setSyslogPriority(c->c_val);
					break;
				}
				c++;
			}
			if (c->c_name == NULL) {
				cout << "Unknown syslog priority";
				return(false);
			}
			continue;
		}
		if (sv[0] == "trap") {
			// trap = [arschloch] $iptables -A arschloch -p ALL -s $ip -j DROP
			split(sv, sv[1], is_any_of("]"));
			string trapLine = sv[1];
			trim(trapLine);
			split(sv,sv[0], is_any_of("["));
			cout << "++ trap name = '" << sv[1] << "' line = '" << trapLine << "'" << endl;
			berzerk->addTrap(sv[1], trapLine);
			continue;
		}
		if (sv[0] == "deflog") {
			// deflog = [logname] /tmp/logfile
			split(sv, sv[1], is_any_of("]"));
			string trapLine = sv[1];
			trim(trapLine);
			split(sv,sv[0], is_any_of("["));
			cout << "++ log name = '" << sv[1] << "' line = '" << trapLine << "'" << endl;
			berzerk->addMappedLogFile(sv[1], trapLine);
			continue;
		}
		if (sv[0] == "include") {
			if (DEBUG > 4)
				cout << "++ include: '" << sv[1] << "'" << endl;
			if (!Filter::parseFile(sv[1].c_str())) {
				return(false);
			}
			continue;
		}

		/////////////////////////////////////////////////////////
		cout << "***Unknown line type '" << sv[0] << "'" << endl;
		return(false);
	}

//	cout << "pattern = '" << this->matchPatterns << "'" << endl;
	if (DEBUG > 3) {
		cout << "  threshold = " << this->threshold << endl;
		cout << "logFmt = '" << this->logFmt << "'" << endl;
//		BOOST_FOREACH(string pat, matchPatterns) {
//			cout << "      pat = '" << pat << "'" << endl;
//		}
	}
	return(true);
}

///////////////////////////////////////////////////////////////////////////////
bool	Config::isWhiteList(const string& ip) {
	BOOST_FOREACH(regex* reg, whitelist) {
		boost::smatch matches;
		if (regex_match(ip, matches, *reg))
			return(true);
	}
	return(false);
}
